@@ -672,7 +672,7 @@ public:
      * Mirror \p dab in the requested direction around the center point defined
      * in the painter. The dab's offset is adjusted automatically.
      */
-    void mirrorDab(Qt::Orientation direction, KisRenderedDab *dab) const;
+    void mirrorDab(Qt::Orientation direction, KisRenderedDab *dab, bool skipMirrorPixels = false) const;
 
     /**
      * Calculate the list of the mirrored rects that will be painted on the
@@ -728,9 +728,7 @@ public:
         FillStyleForegroundColor,
         FillStyleBackgroundColor,
         FillStylePattern,
-        FillStyleGradient,
-        FillStyleStrokes,
-        FillStyleGenerator,
+        FillStyleGenerator
     };
 
     /// Set the current style with which to fill
@@ -739,6 +737,12 @@ public:
     /// Returns the current fill style
     FillStyle fillStyle() const;
 
+    /// Set the transform on the pattern.
+    void setPatternTransform(QTransform transform);
+
+    /// get the current transform on the pattern.
+    QTransform patternTransform();
+
     /// Set whether a polygon's filled area should be anti-aliased or not. The default is true.
     void setAntiAliasPolygonFill(bool antiAliasPolygonFill);
 
