@@ -30,10 +30,15 @@ class KisAutogradientEditor : public QWidget, public Ui::KisWdgAutogradient
     Q_OBJECT
 
 public:
-    KisAutogradientEditor(KoSegmentGradient* gradient, QWidget *parent, const char* name, const QString& caption);
+    KisAutogradientEditor(KoSegmentGradient* gradient, QWidget *parent, const char* name, const QString& caption, KoColor fgColor, KoColor bgColor);
     void activate();
+
+private:
+    void disableTransparentCheckboxes();
+
 private:
     KoSegmentGradient* m_autogradientResource;
+    KoColor m_fgColor, m_bgColor;
 private Q_SLOTS:
     void slotSelectedSegment(KoGradientSegment* segment);
     void slotChangedSegment(KoGradientSegment* segment);
@@ -43,6 +48,11 @@ private Q_SLOTS:
     void slotChangedRightColor(const KoColor& color);
     void slotChangedLeftOpacity(int value);
     void slotChangedRightOpacity(int value);
+    void slotChangedLeftType(QAbstractButton* button, bool checked);
+    void slotChangedRightType(QAbstractButton* button, bool checked);
+    void slotChangedLeftTypeTransparent(bool checked);
+    void slotChangedRightTypeTransparent(bool checked);
+
     void slotChangedName();
     void paramChanged();
 };
