@@ -57,6 +57,8 @@ public:
         return QString("RGBA16");
     }
 
+    void fillGrayBrushWithColorAndLightnessOverlay(quint8 *dst, const QRgb *brush, quint8 *brushColor, qint32 nPixels) const override;
+    void fillGrayBrushWithColorAndLightnessWithStrength(quint8* dst, const QRgb* brush, quint8* brushColor, qreal strength, qint32 nPixels) const override;
 };
 
 class RgbU16ColorSpaceFactory : public LcmsColorSpaceFactory
