@@ -87,13 +87,16 @@ KisToolSelectSimilar::KisToolSelectSimilar(KoCanvasBase * canvas)
                     i18n("Similar Color Selection")),
       m_fuzziness(20)
 {
-    connect(&m_widgetHelper, &KisSelectionToolConfigWidgetHelper::selectionActionChanged,
-            this, &KisToolSelectSimilar::setSelectionAction);
 }
 
 void KisToolSelectSimilar::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
-    KisTool::activate(toolActivation, shapes);
+    KisToolSelect::activate(toolActivation, shapes);
+    if (selectionOptionWidget()) {
+        // similar color selection tool doesn't use antialiasing option for now
+        // hence explicit disabling it
+        selectionOptionWidget()->disableAntiAliasSelectionOption();
+    }
     m_configGroup =  KSharedConfig::openConfig()->group(toolId());
 }
 
@@ -111,6 +114,10 @@ void KisToolSelectSimilar::beginPrimaryAction(KoPointerEvent *event)
         return;
     }
 
+    if (KisToolSelect::selectionDidMove()) {
+        return;
+    }
+
     QPointF pos = convertToPixelCoord(event);
 
     KisCanvas2 * kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
@@ -151,8 +158,9 @@ QWidget* KisToolSelectSimilar::createOptionWidget()
 {
     KisToolSelectBase::createOptionWidget();
     KisSelectionOptions *selectionWidget = selectionOptionWidget();
+    // similar color selection tool doesn't use antialiasing option for now
+    // hence explicit disabling it
     selectionWidget->disableAntiAliasSelectionOption();
-    selectionWidget->disableSelectionModeOption();
 
     QHBoxLayout* fl = new QHBoxLayout();
     QLabel * lbl = new QLabel(i18n("Fuzziness: "), selectionWidget);
@@ -160,7 +168,7 @@ QWidget* KisToolSelectSimilar::createOptionWidget()
 
     KisSliderSpinBox* input = new KisSliderSpinBox(selectionWidget);
     input->setObjectName("fuzziness");
-    input->setRange(0, 200);
+    input->setRange(1, 200);
     input->setSingleStep(10);
     fl->addWidget(input);
     connect(input, SIGNAL(valueChanged(int)), this, SLOT(slotSetFuzziness(int)));
@@ -174,16 +182,13 @@ QWidget* KisToolSelectSimilar::createOptionWidget()
     return selectionWidget;
 }
 
-void KisToolSelectSimilar::setSelectionAction(int action)
+void KisToolSelectSimilar::resetCursorStyle()
 {
-    changeSelectionAction(action);
-}
-
-QMenu* KisToolSelectSimilar::popupActionsMenu()
-{
-    KisCanvas2 * kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
-    Q_ASSERT(kisCanvas);
-
-
-    return KisSelectionToolHelper::getSelectionContextMenu(kisCanvas);
+    if (selectionAction() == SELECTION_ADD) {
+        useCursor(KisCursor::load("tool_similar_selection_cursor_add.png", 6, 6));
+    } else if (selectionAction() == SELECTION_SUBTRACT) {
+        useCursor(KisCursor::load("tool_similar_selection_cursor_sub.png", 6, 6));
+    } else {
+        KisToolSelect::resetCursorStyle();
+    }
 }
